#!/usr/bin/php
<?php
/**
 * Simple page by email tool
 *
 * It reads an email from STDIN, parses it and creates/updates
 * the page given in the subject
 *
 * Attachments are stored in media
 */
if ('cli' != php_sapi_name()) die();
if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../').'/');
require_once DOKU_INC.'inc/init.php';
require_once DOKU_INC.'inc/common.php';
require_once DOKU_INC.'inc/mime_parser.php';
if(!defined('NL')) define('NL',"\n");

// read the mail and parse it
$data = array();
$mail = new mime_parser_class();
$ok = $mail->Decode(array('File' => 'php://stdin'), $data);
if(!$ok) die('decoding failed');
$data = $data[0];

#print_r($data);

// set up the reporting address
$from = m2p_getHeader($data,array('reply-to:','from:'));

// check the subject for the page id
$ID = cleanID(m2p_getHeader($data,'subject:'));
if(!$ID) m2p_finish('No ID given');
msg("Saving to $ID");

// check the parts
if(count($data['Parts']) == 0){
    msg('Single-Part message');
    if(substr($data['Headers']['content-type:'],0,10) != 'text/plain'){
        m2p_finish('bad content type - we can only handle text/plain');
    }
    m2p_handlePage($data['Body'], $data['Headers']['content-type:']);
}else{
    msg('Multi-Part message');
    foreach($data['Parts'] as $part){
        msg('Found: '.$part['Headers']['content-type:']);
        if(substr($part['Headers']['content-type:'],0,10) == 'text/plain' &&
           $part['Headers']['content-disposition:'] != 'attachment'){
            m2p_handlePage($part['Body'],$data['headers']['content-type:']);
        }else{
            m2p_handleMedia($part['Body'],$part['FileName']);
        }
    }
}

m2p_finish();

// ---------------------------------------------------------------------

/**
 * Stores attachments in the media directory
 */
function m2p_handleMedia(&$body,$fname){
    global $ID;
    global $conf;
    global $lang;

    if(!$fname) return;
    msg('Handling '.$fname);

    $fid = cleanID(getNS($ID).':'.$fname);
    list($iext,$imime) = mimetype($fid);
    $fn   = mediaFN($fid);
    msg('Saving to '.$fid);

    // get filetype regexp
    require_once(DOKU_INC.'inc/media.php');
    $types = array_keys(getMimeTypes());
    $types = array_map(create_function('$q','return preg_quote($q,"/");'),$types);
    $regex = join('|',$types);

    if(!preg_match('/\.('.$regex.')$/i',$fn)){
        msg($lang['uploadwrong'],-1);
        return;
    }

    io_createNamespace($fid, 'media');
    if(io_saveFile($fn,$body)){
        chmod($fn, $conf['fmode']);
        msg($lang['uploadsucc'],1);
        if(function_exists('media_notify')){
            media_notify($fid,$fn,$imime);
        }
    }
}

/**
 * Convert the body charset and save the page
 */
function m2p_handlePage(&$body,$ctype){
    global $ID;

    static $run = false;
    if($run){
        msg('Second page body found - ignored');
        return;
    }
    $run = true;

    msg('Handling body');

    if(preg_match('/charset=["\']?([^"\']+)["\';]?$/i',$ctype,$match)){
        msg('Converting from charset '.$match[1]);
        $body = iconv($match[1],'utf-8',$body);
    }

    $body = cleanText($body);
    //strip signature:
    $body = preg_replace('/\n?-- (.*)$/s','',$body);
    msg('Body length: '.strlen($body).' bytes');

    //save page
    saveWikiText($ID,$body,'submitted by email');
}

/**
 * Get a (maybe decoded) header from any of the given fields
 */
function m2p_getHeader(&$data,$headers){
    if(!is_array($headers)) $headers = array($headers);

    $value = '';
    foreach($headers as $header){
        if($data['DecodedHeaders'][$header][0][0]['Value']){
            $value = iconv($data['DecodedHeaders'][$header][0][0]['Encoding'],
                    'utf-8', $data['DecodedHeaders'][$header][0][0]['Value']);
            break;
        }
        if($data['Headers'][$header]){
            $value = $data['Headers'][$header];
            break;
        }
    }
    return $value;
}

/**
 * Send a report back and end the script
 *
 * If $message is set it is added to the report before sending it.
 */
function m2p_finish($message=''){
    global $from;
    global $ID;
    global $MSG;
    global $conf;
    if($message) msg($message);

    $subject = '['.$conf['title'].'] Your email submission';
    $text = 'The following reports were created while processing your submission'.NL.NL;
    foreach($MSG as $m){
        $text .= $m['msg'].NL;
    }

    echo $text;
    flush();

    if(!$from){
        die('no reply address');
    }
    mail_send($from, $subject, $text, $conf['mailfrom']);
    exit();
}
